#include "Driver_ArduinoNano.h"
#include "platform_config.h"
#include <Arduino.h>

//pin_nCS	PB2 SS D12
#define	pin_nCS_bit		4
#define	pin_nCS_DDR		DDRB
#define	pin_nCS_PORT	PORTB

//pin_led PB1 D9
#define	pin_led_bit		1
#define	pin_led_DDR		DDRB
#define	pin_led_PORT	PORTB
//-------------------------------

//pin_nClock PB5 (SCK) D13
//#define	pin_nClock_bit	5
//#define	pin_nClock_DDR	DDRB
//#define	pin_nClock_PORT	PORTB

//pin_nClock PB5 (SCK) D10
#define	pin_nClock_bit	2
#define	pin_nClock_DDR	DDRB
#define	pin_nClock_PORT	PORTB

//PB3 MOSI D11
#define	pin_SDIO_bit		3
#define	pin_SDIO_DDR		DDRB
#define	pin_SDIO_PORT		PORTB
#define	pin_SDIO_PORT_IN	PINB

#define	pin_nCS_Mode_OUTPUT		(pin_nCS_DDR			|= (1<<pin_nCS_bit))
#define	pin_nCS_LOW				(pin_nCS_PORT		&=~(1<<pin_nCS_bit))
#define	pin_nCS_HIGH			(pin_nCS_PORT		|= (1<<pin_nCS_bit))

#define	pin_led_Mode_OUTPUT		(pin_led_DDR	|= (1<<pin_led_bit))
#define	pin_led_HIGH			(pin_led_PORT	&=~(1<<pin_led_bit))
#define	pin_led_LOW		    	(pin_led_PORT	|=(1<<pin_led_bit))
#define	pin_led_Toggle			(pin_led_PORT	^=(1<<pin_led_bit))

#define	pin_nClock_Mode_OUTPUT	(pin_nClock_DDR		|= (1<<pin_nClock_bit))
#define	pin_nClock_LOW			(pin_nClock_PORT		&=~(1<<pin_nClock_bit))
#define	pin_nClock_HIGH			(pin_nClock_PORT		|=(1<<pin_nClock_bit))

#define	pin_SDIO_Mode_INPUT		(pin_SDIO_DDR		&=~(1<<pin_SDIO_bit))
#define	pin_SDIO_Mode_OUTPUT	(pin_SDIO_DDR		|= (1<<pin_SDIO_bit))
#define	pin_SDIO_LOW			(pin_SDIO_PORT		&=~(1<<pin_SDIO_bit))
#define	pin_SDIO_HIGH			(pin_SDIO_PORT		|= 1<<pin_SDIO_bit)
#define	pin_SDIO_IN				(pin_SDIO_PORT_IN	&  (1<<pin_SDIO_bit))

void configurePinAsOutput(int pin) {
  pinMode(pin, OUTPUT);
}

void configurePinAsInput(int pin) {
  pinMode(pin, INPUT);
}

void setPin(int pin) {
  digitalWrite(pin, HIGH);
}

void resetPin(int pin) {
  digitalWrite(pin, LOW);
}

void SPI_init(void) {
#ifdef SPI_SOFTWARE
    pin_nClock_Mode_OUTPUT;
    pin_nClock_HIGH;
    pin_SDIO_Mode_OUTPUT;
    pin_SDIO_LOW;
#else
    SPI.begin();
    SPI.setDataMode(SPI_MODE3);
#endif // SPI_SOFTWARE
}

void SPI_nCS_LOW(void) {
    pin_nCS_LOW;
}

void SPI_nCS_HIGH(void) {
    pin_nCS_HIGH;
}

void SPI_SDIO_Mode_INPUT(void) {
    pin_SDIO_Mode_INPUT;
}

void SPI_SDIO_Mode_OUTPUT(void) {
    pin_SDIO_Mode_OUTPUT;
}

void SPI_write(uint8_t data)
{
#ifdef SPI_SOFTWARE
    uint8_t i;
    for(i=0x80; i; i>>=1)
    {
        pin_nClock_LOW;
        data & i ? pin_SDIO_HIGH : pin_SDIO_LOW;
        asm volatile ("nop");
        pin_nClock_HIGH;
    }
#else
    SPI.transfer(data);
#endif // SPI_SOFTWARE
}

uint8_t SPI_receive(void) {
    #ifdef SPI_SOFTWARE
    uint8_t data = 0;
    uint8_t i;
	for (i = 8; i; i--)
	{
		// tick, tock, read
		pin_nClock_LOW;
		asm volatile ("nop");
		pin_nClock_HIGH;
		data <<= 1;
		if(pin_SDIO_IN) data |= 0x01;
	}
#else
    SPI.transfer(data);
#endif // SPI_SOFTWARE
}

void delay_ms(uint32_t ms) {
    delay(ms);
}

void delay_us(uint16_t us) {
    delayMicroseconds(us);
}
